// TCO-Mini_Reseau_V5.ino
// ----------------------------------------------------------
// C. BEZANGER - 24 juillet 2017
// Version 4 - 27 juillet 2017
// Version 5 - 21 octobre 2017 (fonctionnel)
// Ce programme fonctionne en complement du programme 
// Processing TCO_Mini_Reseau_V2.pde
// Il receptionne les etats de l'aiguilles A et
// allume la LED_BUILTIN affectee a l'aiguille si deviee
// Il modifie l etat de l aiguille A avec un poussoir
// et envoie cet etat a Processing pour mise a jour du TCO
// Il reçoit egalement les sens de circulation sur la voie
// terminus et sur la boucle et envoie la bonne polarite aux
// rails
// Un capteur indique quand inverser le courant et le programme 
// envoyer les nouvelles donnees au TCO
// La version 3 prend en charge le servomoteur qui deplace 
// l aiguille
// La version 4 delivre sur les sorties 8 et 9 le meme signal 
// qu en 11 et 12 pour commander les relais inverseurs
// La version 5 est identique a la 4 mais avec des switch case
// au lieu des series de if
//-----------------------------------------------------------

#include <Servo.h>  // utilise la bibliotheque Servo
Servo servo_aiguilleA; // crée un servo pour l aiguille A

const char HEADER = 'H';
const int TOTAL_BYTES = 3;// nombre de caracteres envoyes ou recus
const byte bouton_A = 2; // B/P actionnant l aiguille A
const byte bouton_L = 3; // B/P capteur sur boucle

char caractere2; // deuxieme caractere lu
char caractere3; // troisieme caracter lu
int sensVT = 1; // sens de circulation sur voie terminus
int sensB = 1; // sens de circulation sur boucle
boolean bscA = false; // aiguille A non deviee
boolean oldbscA = false; // derniere position aiguille en memoire

void setup() {
  Serial.begin(9600); // initialise la transmission serie
  servo_aiguilleA.attach(10); // attache le servo a la broche 10
  pinMode(bouton_A, INPUT_PULLUP);  // relie a la masse GND
  pinMode(bouton_L, INPUT_PULLUP);  // relie a la masse GND
  // interruption sur broches 2 et 3 front descendant
  attachInterrupt(digitalPinToInterrupt(bouton_A),appuiBoutonA,FALLING);
  attachInterrupt(digitalPinToInterrupt(bouton_L),appuiBoutonL,FALLING);
  // les sorties 13,12,11
  pinMode(LED_BUILTIN, OUTPUT); // visualise l aiguille A
  pinMode(12, OUTPUT);  // visualise le sens de circulation sur voie terminus
                        // allume si sens depart
  pinMode(11, OUTPUT);  // visualise le sens de circulation sur la boucle
                        // allume si sens des aiguilles d une montre
  pinMode (9,OUTPUT);
  pinMode(8, OUTPUT);
  digitalWrite(LED_BUILTIN, LOW);
  digitalWrite(12, LOW);
  digitalWrite(11, LOW);
}

void loop() {
    // recopie des signaux 11 et 12
    digitalWrite(8, digitalRead(11));
    digitalWrite(9, digitalRead(12));

    // Positionnement de l aiguille si necessaire
    if(bscA != oldbscA) // il y a eu changement de position de l aiguille
    {
      aiguillage(bscA);
      oldbscA = bscA;      
    }

    // reception des informations envoyees par le TCO
    if(Serial.available() >= TOTAL_BYTES) // au moins 3 caracteres disponibles
    {
      if(Serial.read() == HEADER) // est-ce bien l entete ?
      {
        // Oui donc lecture des deux caracteres suivants
        caractere2 = Serial.read();
        caractere3 = Serial.read();
        // Etude des differentts cas pour les deux caracteres
        switch (caractere2) {
          case 'A' :  // On parle de l aiguille A
          switch (caractere3) {
            case '1' :
            bscA = true;
            break;
            case '0' :
            bscA = false;
            break;
          }
          break;
          case 'T' :  // On parle de la voie terminus
          switch (caractere3) {
            case '0' :
            sensVT = 0;
            digitalWrite(12, HIGH);
            break;
            case '1' :
            sensVT = 1;
            digitalWrite(12, LOW);
            break;
          }
          break;
          case 'L' :  // On parle de la boucle (Loop)
          switch (caractere3) {
            case '0' :
            sensB = 0;
            digitalWrite(11, HIGH);
            break;
            case '1' :
            sensB = 1;
            digitalWrite(11, LOW);
            break;
          }
          break;
        }
      }
    }
}

void appuiBoutonA()
{
  bscA = !bscA;
    if(!bscA && sensVT == 1) {sensB = 1;}
    if(!bscA && sensVT == 0) {sensB = 0;}
    if(bscA && sensVT == 1) {sensB = 0;}
    if(bscA && sensVT == 0) {sensB = 1;}
    // communication vers TCO
    if(bscA) {Serial.print(HEADER);Serial.print('A');Serial.print('1');digitalWrite(LED_BUILTIN, HIGH);}
    else {Serial.print(HEADER);Serial.print('A');Serial.print('0');digitalWrite(LED_BUILTIN, LOW);}
    if(sensB == 0) {Serial.print(HEADER);Serial.print('L');Serial.print('0');}
    if(sensB == 1) {Serial.print(HEADER);Serial.print('L');Serial.print('1');}
}

void appuiBoutonL()
{
  bscA = !bscA; // on met l aiguille dans le sens retour
  if(bscA){digitalWrite(LED_BUILTIN, HIGH);}
  if(!bscA) {digitalWrite(LED_BUILTIN, LOW);}
  // communication vers TCO
  if(bscA) {Serial.print(HEADER);Serial.print('A');Serial.print('1');}
  if(!bscA) {Serial.print(HEADER);Serial.print('A');Serial.print('0');}
  
  sensVT = 1; // permet le retour sur voie terminus
  // communication vers TCO
  Serial.print(HEADER);
  Serial.print('T');
  Serial.print('1');
  digitalWrite(12, LOW);
}

void aiguillage(boolean bsc)  // deplace l aiguille A
{
  int butee_non_deviee = 1300; // a regler en fonction de l aiguille
  int butee_deviee = 1700; // a regler en fonction de l aiguille
  int pos_intermediaire; // position durant le transit
  int pas = 1; // permet de regler la fluidite du mouvement
  int attente = 10; // permet de regler la vitesse du mouvement  

  if(bsc)  // il faut mettre le servomoteur en position aiguille deviee
  {
    pos_intermediaire = butee_non_deviee;
    while(pos_intermediaire <= butee_deviee)
    {
      servo_aiguilleA.writeMicroseconds(pos_intermediaire);
      pos_intermediaire = pos_intermediaire + pas;
      delay(attente);
      if((pos_intermediaire % 25) == 0)
      {
        digitalWrite(LED_BUILTIN, !digitalRead(LED_BUILTIN));    
      }
    }
    digitalWrite(LED_BUILTIN, HIGH);
  }
  if(!bsc)  // il faut mettre le servomoteur en position aiguille non deviee
  {
    pos_intermediaire = butee_deviee;
    while(pos_intermediaire >= butee_non_deviee)
    {
      servo_aiguilleA.writeMicroseconds(pos_intermediaire);
      pos_intermediaire = pos_intermediaire - pas;
      delay(attente);
      if((pos_intermediaire % 25) == 0)
      {
        digitalWrite(LED_BUILTIN, !digitalRead(LED_BUILTIN));    
      }    
    }
    digitalWrite(LED_BUILTIN, LOW);
  }
}


