// Construction d'un TCO pour mini réseau - voie terminus et boucle retournement
//---------------------------------------------------------------------------------
// C. BEZANGER - 24 juillet 2017
// Le TCO est construit sur une grille de 19 colonnes et 5 lignes
// Lignes et colonnes sont numérotées à partir de 0, l'aiguille "A" est en (10, 2)
// La voie terminus s'appelle voie 1
// Version 2 - 25 juillet 2017 - Prend en compte le sens de circulation
// 31 juillet 2017 - Affichage DEP ou RET sur le butoir, et des appellations
// Le changement de sens DEP ou RET se fait en cliquant sur le butoir

import processing.pdf.*; // permet d'afficher du texte
import processing.serial.*; // permet la communication série avec Arduino

String portArduino = "COM4"; // Mettre ici le port sur lequel est connecté Arduino
Serial myPort; // Crée un objet à partir de la classe Serial

int GRIS=#7F7F7F,NOIR=#000000,BLANC=#FFFFFF,COULEUR_NEGATIF=#00FF00,COULEUR_POSITIF=#FF7F00;
int zoom=10;  // réglable de 1 à 10, selon l'écran
int J; // indice de ligne
int TOTAL_BYTES = 3; // nombre total de caractères pouvant être reçus
int caractere2; // valeur reçue pour deuxième caractère
int caractere3; // valeur reçue pour troisième caractère
int sensVT=1; // sens de parcours de la voie terminus, 0 sens départ, 1 sens arrivée
int sensB=1; // sens parcours de la boucle, 1 sens des aiguilles d'une montre, 0 sinon
boolean bscA = false; // bascule aiguille A, true si déviée à droite
char HEADER = 'H'; // permet de définir le début d'une communication

PShape barre;    // barre diagonale /
PShape A_barre;  // anti barre diagonale \
PShape arc0;     // voie courbe gauche vers en haut à droite
PShape arc1;     // voie courbe en haut à gauche --> droite
PShape arc2;     // voie courbe gauche --> en bas à droite
PShape arc3;     // voie courbe en bas à gauche --> droite
PShape arc4;     // voie courbe bas --> en haut à gauche
PShape arc5;     // voie courbe haut --> en bas à droite

/*========================================================================================
 * ouvrirConnexionArduino ouvre une connexion série après avoir vérifié que le port
 * demandé existe bien.
 */
Serial ouvrirConnexionArduino(String inPortName)
{
  String[] portsDisponibles = Serial.list();
  boolean portOk = false;
  for (int i = 0; i < portsDisponibles.length; i++) {
    if (portsDisponibles[i].equals(inPortName)) {
      portOk = true;
      break;
    }
  }
  if (portOk) {
    return new Serial(this, inPortName, 9600);
  }
  else {
    return null;
  }
}

void setup() {
  //size(1900,1000,P3D); // taille de la fenêtre d'affichage
  fullScreen(P3D); // cette ligne ou la précédente, mais pas les deux
  textMode(MODEL);
  textSize(3*zoom);
    
  // Connexion avec Arduino
  String portName = portArduino;
  // printArray(Serial.list()); // à décommenter pour trouver la liste des ports série
  println("Connexion à -> "+portName);
  myPort = ouvrirConnexionArduino(portName);
  
  if (myPort != null) {
    println("l : "+width+"  h: "+height);
    
    // création des formes pour dessiner les pavés
    barre=createShape(); // creation de la forme barre
    barre.beginShape(); // debut du contour de la forme
    barre.noStroke(); // pas de dessin du contour
    barre.vertex(8*zoom,-1*zoom); // point de depart
    barre.vertex(10*zoom,1*zoom);
    barre.vertex(1*zoom,10*zoom);
    barre.vertex(-1*zoom,8*zoom);
    barre.endShape(CLOSE); // fermeture du contour de la forme (ligne droite)
    
    A_barre=createShape(); // creation de la forme A_barre
    A_barre.beginShape(); // debut du contour de la forme
    A_barre.noStroke(); // pas de dessin du contour
    A_barre.vertex(-1*zoom,1*zoom); // point de depart
    A_barre.vertex(1*zoom,-1*zoom); // ligne droite
    A_barre.vertex(10*zoom,8*zoom); // ligne droite
    A_barre.vertex(8*zoom,10*zoom); // ligne droite
    A_barre.endShape(CLOSE); // fermeture du contour de la forme (ligne droite)
    
    arc0=createShape(); // creation de la forme
    arc0.beginShape(); // debut du contour de la forme
    arc0.noStroke(); // pas de dessin du contour
    arc0.vertex(0*zoom,3*zoom); // point de depart
    arc0.quadraticVertex(4*zoom,3*zoom, 8*zoom,-1*zoom); // courbe Bezier quadratique
    arc0.vertex(10*zoom,1*zoom); // ligne droite
    arc0.quadraticVertex(5*zoom,6*zoom, 0*zoom,6*zoom);  // courbe Bezier quadratique
    arc0.endShape(CLOSE); // fermeture du contour de la forme (ligne droite)
    
    arc1=createShape(); // creation de la forme
    arc1.beginShape(); // debut du contour de la forme
    arc1.noStroke(); // pas de dessin du contour
    arc1.vertex(1*zoom,-1*zoom); // point de depart
    arc1.quadraticVertex(5*zoom,4*zoom, 9*zoom,3*zoom); // courbe Bezier quadratique
    arc1.vertex(9*zoom,6*zoom); // ligne droite
    arc1.quadraticVertex(4*zoom,6*zoom, -1*zoom,1*zoom);  // courbe Bezier quadratique
    arc1.endShape(CLOSE); // fermeture du contour de la forme (ligne droite)
  
    arc2=createShape(); // creation de la forme
    arc2.beginShape(); // debut du contour de la forme
    arc2.noStroke(); // pas de dessin du contour
    arc2.vertex(0*zoom,3*zoom); // point de depart
    arc2.quadraticVertex(5*zoom,3*zoom,10*zoom,8*zoom); // courbe Bezier quadratique
    arc2.vertex(8*zoom,10*zoom); // ligne droite
    arc2.quadraticVertex(4*zoom,6*zoom,0*zoom,6*zoom);  // courbe Bezier quadratique
    arc2.endShape(CLOSE); // fermeture du contour de la forme (ligne droite)
    
    arc3=createShape(); // creation de la forme
    arc3.beginShape(); // debut du contour de la forme
    arc3.noStroke(); // pas de dessin du contour
    arc3.vertex(-1*zoom,8*zoom); // point de depart
    arc3.quadraticVertex(4*zoom,3*zoom,9*zoom,3*zoom); // courbe Bezier quadratique
    arc3.vertex(9*zoom,6*zoom); // ligne droite
    arc3.quadraticVertex(5*zoom,6*zoom,1*zoom,10*zoom);  // courbe Bezier quadratique
    arc3.endShape(CLOSE); // fermeture du contour de la forme (ligne droite)
    
    arc4=createShape(); // creation de la forme
    arc4.beginShape(); // debut du contour de la forme
    arc4.noStroke(); // pas de dessin du contour
    arc4.vertex(3*zoom,9*zoom); // point de depart
    arc4.quadraticVertex(3*zoom,5*zoom,-1*zoom,1*zoom); // courbe Bezier quadratique
    arc4.vertex(1*zoom,-1*zoom); // ligne droite
    arc4.quadraticVertex(6*zoom,4*zoom, 6*zoom,9*zoom);  // courbe Bezier quadratique
    arc4.endShape(CLOSE); // fermeture du contour de la forme (ligne droite)
    
    arc5=createShape(); // creation de la forme
    arc5.beginShape(); // debut du contour de la forme
    arc5.noStroke(); // pas de dessin du contour
    arc5.vertex(3*zoom,0*zoom); // point de depart
    arc5.quadraticVertex(3*zoom,4*zoom, -1*zoom,8*zoom); // courbe Bezier quadratique
    arc5.vertex(1*zoom,10*zoom); // ligne droite
    arc5.quadraticVertex(6*zoom,5*zoom, 6*zoom,0*zoom);  // courbe Bezier quadratique
    arc5.endShape(CLOSE); // fermeture du contour de la forme (ligne droite)
  }
  else {
    println("Le port " + portName + " n'est pas disponible");
    exit();
  }
}

void draw() {
  // surveillance des données entrantes
  if(myPort.available() >= TOTAL_BYTES) // au moins 3 caractères disponibles
  {
    if(myPort.read() == HEADER) // est-ce bien l'entête ?
    {
      // oui, on peut lire les deux autres caractères
      caractere2 = myPort.read();
      caractere3 = myPort.read();
      if(caractere2 == 'A') // on parle de l'aiguille A
      {
        if(caractere3 == '0') {bscA = false;}
        if(caractere3 == '1') {bscA = true;}
      }
      if(caractere2 == 'T') // on parle de la voie terminus
      {
        if(caractere3 == '0') {sensVT = 0;}
        if(caractere3 == '1') {sensVT = 1;}
      }
      if(caractere2 == 'L') // on parle de la boucle
      {
        if(caractere3 == '0') {sensB = 0;}
        if(caractere3 == '1') {sensB = 1;}
      }
    }
  }
  
  // construction du TCO ligne par ligne
  // LIGNE 0
  J = 0;
  for(int i = 0; i <= 11; i++)
  {
    pave_vide(i*9,J*9);
  }
  pave_arc3(12*9,J*9,sensB);
  for(int i = 13; i <= 16; i++)
  {
    pave_droit_H(i*9,J*9,sensB);
  }
  pave_arc2(17*9,J*9,sensB);
  pave_vide(18*9,J*9);
  
  // LIGNE 1
  J = 1;
  for(int i = 0; i <= 10; i++)
  {
    pave_vide(i*9,J*9);
  }
  pave_barre(11*9,J*9,sensB);
  for(int i = 12; i <= 17; i++)
  {
    pave_vide(i*9,J*9);
  }
  pave_arc4(18*9,J*9,sensB);
  
  // LIGNE 2
  J = 2;
  pave_butoir_G(0*9,J*9);
  for(int i = 1; i <= 9; i++)
  {
    pave_droit_H(i*9,J*9,sensVT);
  }
  if(sensVT == 1)
  {
    for(int i = 1; i <= 3; i++)
    {
      pave_droit_H(i*9,J*9,2); // 2 indique voie non alimentée
    }
  }
  pave_aiguille(10*9,J*9,bscA,sensVT);
  for(int i = 11; i <= 17; i++)
  {
    pave_vide(i*9,J*9);
  }
  pave_droit_V(18*9,J*9);
  
  // LIGNE 3
  J = 3;
  for(int i = 0; i <= 10; i++)
  {
    pave_vide(i*9,J*9);
  }
  pave_anti_barre(11*9,J*9,1-sensB);
  for(int i = 12; i <= 17; i++)
  {
    pave_vide(i*9,J*9);
  }
  pave_arc5(18*9,J*9,1-sensB);
  
  // LIGNE 4
  J = 4;
  for(int i = 0; i <= 11; i++)
  {
    pave_vide(i*9,J*9);
  }
  pave_arc1(12*9,J*9,1-sensB);
  for(int i = 13; i <= 16; i++)
  {
    pave_droit_H(i*9,J*9,1-sensB);
  }
  pave_arc0(17*9,J*9,1-sensB);
  pave_vide(18*9,J*9);
  
  // Affichage de DEP ou RET sur le butoir, du numéro de voie et du nom de l'aiguille
  fill(255,255,255);
  if(sensVT == 0) {text("DEP",3*zoom,23.5*zoom);}
  if(sensVT == 1) {text("RET",3*zoom,23.5*zoom);}
  fill(255,255,0);
  text("A",93*zoom,17.5*zoom);
  fill(0,0,0);
  text("1",48*zoom,23.5*zoom);
  
}

void mouseClicked()
{
  if(mouseX >= 90*zoom && mouseX <= 99*zoom && mouseY >= 18*zoom && mouseY <= 27*zoom)
  // on vient de cliquer sur le pavé "aiguille"
  {
    bscA = !bscA; // changement de l'état de déviation
    // redéfinition du sens de circulation sur la boucle (idem à voie terminus)
    if(!bscA && sensVT == 1) {sensB = 1;}
    if(!bscA && sensVT == 0) {sensB = 0;}
    if(bscA && sensVT == 1) {sensB = 0;}
    if(bscA && sensVT == 0) {sensB = 1;}
    // communication vers Arduino
    if(bscA) {myPort.write(HEADER);myPort.write("A");myPort.write("1");}
    else {myPort.write(HEADER);myPort.write("A");myPort.write("0");}
    if(sensB == 0) {myPort.write(HEADER);myPort.write("L");myPort.write("0");}
    if(sensB == 1) {myPort.write(HEADER);myPort.write("L");myPort.write("1");}
  }
  if(mouseX >= 0*zoom && mouseX <= 9*zoom && mouseY >= 18*zoom && mouseY <= 27*zoom)
  // on vient de cliquer sur le butoir
  {
    sensVT = 1 - sensVT;
    sensB = 1 - sensB;
    if(sensVT == 0) {myPort.write(HEADER);myPort.write("T");myPort.write("0");}
    if(sensVT == 1) {myPort.write(HEADER);myPort.write("T");myPort.write("1");}
    if(sensB == 0) {myPort.write(HEADER);myPort.write("L");myPort.write("0");}
    if(sensB == 1) {myPort.write(HEADER);myPort.write("L");myPort.write("1");}
  }
}